'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = resolve;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fse = require('fse');

var _fse2 = _interopRequireDefault(_fse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function resolve(id, rawcwd) {
	// set cwd as the current working directory
	var cwd = rawcwd;

	// if id begins with /
	if (id[0] === '/') {
		// set cwd as the filesystem root
		cwd = '';
	}

	// resolve load_as_file(cwd/id)
	return load_as_file(_path2.default.join(cwd, id)).catch(
	// otherwise, resolve load_as_directory(cwd/id)
	function () {
		return load_as_directory(_path2.default.join(cwd, id));
	}).catch(function () {
		return load_node_modules(id, _path2.default.dirname(cwd));
	}).catch(function () {
		return Promise.reject('CSS Module not found');
	});
}

function load_as_file(id) {
	// resolve if id is a file
	return is_file(id).catch(
	// otherwise, resolve if id.css is a file
	function () {
		return is_file(id + '.css');
	});
}

function load_as_directory(id) {
	// if id/package.json is a file
	return is_file(_path2.default.join(id, 'package.json')).then(
	// parse id/package.json as pkg
	function () {
		return _fse2.default.readFile(_path2.default.join(id, 'package.json')).then(function (json) {
			return JSON.parse(json);
		});
	}).then(
	// if style field exists in pkg
	function (pkg) {
		return 'style' in pkg
		// resolve load_as_file(id/pkg.style)
		? load_as_file(_path2.default.join(id, pkg.style))
		// otherwise, resolve load_index(id)
		: load_index(id);
	});
}

function load_node_modules(id, start) {
	return node_modules_paths(start).reduce(
	// for each dir in node_modules_paths(start):
	function (promise, dir) {
		return promise.catch(
		// resolve load_as_file(dir/id)
		function () {
			return load_as_file(_path2.default.join(dir, id)).catch(
			// resolve load_as_directory(dir/id)
			function () {
				return load_as_directory(_path2.default.join(dir, id));
			});
		});
	}, Promise.reject());
}

function node_modules_paths(start) {
	// set parts as the split paths of start
	var parts = start.split(_path2.default.sep);

	// set i as the count of parts
	var i = parts.length;

	// set dirs as empty array
	var dirs = [];

	// while i >= 0
	while (i > 0) {
		// if parts i is node_modules
		if (parts[i] === 'node_modules') {
			// continue
			continue;
		}

		// push joined paths of parts 0 through i and node_modules to dirs
		dirs.push(_path2.default.join.apply(_path2.default, _toConsumableArray(parts.slice(0, i)).concat(['node_modules'])));

		// set i as i minus 1
		--i;
	}

	// return dirs
	return dirs;
}

function load_index(id) {
	// resolve if id/index.css is a file
	return is_file(_path2.default.join(id, 'index.css'));
}

function is_file(id) {
	return _fse2.default.lstat(id).then(function (stat) {
		return stat.isFile() ? Promise.resolve(id) : Promise.reject();
	});
}
